<?PHP
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of Travelsized CMS
 *              A content management system with modules, based on wiki syntax
 *
 * Author: Dan Jensen <admin@leinir.dk>
 * Copyright 2003/2004
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The GNU General Public License is available at: http://www.gnu.org/copyleft/
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

if (!function_exists("file_get_contents")) {
	function file_get_contents($filename) {
		if (file_exists($filename)) {
			$fp = fopen($filename, "r");
			$contents = fread($fp, filesize($filename));
			fclose($fp);
			return $contents;
		} else {
			return false;
		}
	}
}

if (!function_exists("file_put_contents")) {
	function file_put_contents($filename, $content) {
		touch($filename);
		if (is_writable($filename)) {
			if (!$handle = fopen($filename, 'a')) {
				return false;
			}
			// Write $somecontent to our opened file.
			if (fwrite($handle, $content) === FALSE) {
				return false;
			}
			
			fclose($handle);
			return true;
			
		} else {
			return false;
		}
	}
}

function array_export($array_to_export, $name_of_array) {
	$encoded = "<?PHP
\$$name_of_array = " . var_export($array_to_export, true) . ";
?>";
	return $encoded;
}

/**
 * Delete a file, or a folder and its contents
 *
 * @author      Aidan Lister <aidan@php.net>
 * @version     1.0.1
 * @param       string   $dirname    Directory to delete
 * @return      bool     Returns TRUE on success, FALSE on failure
 */
function rmdirr($dirname)
{
    // Sanity check
    if (!file_exists($dirname)) {
        return false;
    }

    // Simple delete for a file
    if (is_file($dirname)) {
        return unlink($dirname);
    }

    // Loop through the folder
    $dir = dir($dirname);
    while (false !== $entry = $dir->read()) {
        // Skip pointers
        if ($entry == '.' || $entry == '..') {
            continue;
        }

        // Deep delete directories      
        if (is_dir("$dirname/$entry")) {
            rmdirr("$dirname/$entry");
        } else {
            unlink("$dirname/$entry");
        }
    }

    // Clean up
    $dir->close();
    return rmdir($dirname);
}

/**
 * Delete files with a wildcard
 */
function unlink_multiple($str) { 
	foreach(glob($str) as $fn) {
		if (!unlink($fn)) return false;
	}
	return true;
}

/**
 * Move files containing with names containing the string $needle
 * from the folder $from to the folder $target
 */
function rename_multiple($from, $target, $needle) {
	$dir = dir($from);
	while (false !== $entry = $dir->read()) {
		// Skip pointers, and files not containing the needle
		if ($entry == '.' || $entry == '..' || strpos($entry, $needle) === false) {
			continue;
		} else {
			if (!rename("$from/$entry", "$target/$entry")) return false;
		}
	}
	$dir->close();
	return true;
}


/**
 * RecursiveMkdir function for generating new folders recursively
 * Courtesy of David Pullen in the comments in the PHP manual on PHP.net
 */
function RecursiveMkdir($path) {
	// This function creates the specified directory using mkdir().  Note
	// that the recursive feature on mkdir() is broken with PHP 5.0.4 for
	// Windows, so I have to do the recursion myself.
	if (!file_exists($path))
	{
		// The directory doesn't exist.  Recurse, passing in the parent
		// directory so that it gets created.
		RecursiveMkdir(dirname($path));
		
		mkdir($path, 0775);
	}
	return true;
}

/**
 * Make a random password, the usable characters are defined by the salt value
 * courtesy of Ian Carter on www.phpfreaks.com
 */
function makeRandomPassword($n=8, $chars='abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890') {
	$m=strlen($chars);
	while($n--) {
		$x.=substr($chars,rand()%$m,1);
	}
	return $x;
}

/**
 * Check for a valid email address
 * courtesy of Patrick and Sven in the php.net manual
 */
function validateEmail($email){
	$exp = ".+@.+\..+.";
	if(eregi($exp,$email)){
		$exploded = explode("@",$email);
		$popped = array_pop($exploded);
		if(checkdnsrr($popped,"MX")){
			return true;
		}else{
			return false;
		}
	} else {
		return false;
	}
}

/**
 * Str Pos Nth (Position of nth occurance of a string)
 * A handy function to get the position of nth occurance of a substring in a string, with an optional param to make it case insenstive. I am calling it strposnth, suggestions welcome.
 *
 * Third optional parameter gets the value of n, e.g puting in 2 will return position of second occurance of needle in haystack: Valid inputs (1 = default) 2,3,4..... 
 *
 * Fourth optional parameter can be used to specify the function as case insenstive: Valid inputs (0 = case senstive = default) 1 = case insenstive.
 *
 * Courtesy of webKami [at] akdomains.com in the php.net manual
 */

function strposnth($haystack, $needle, $nth=1, $insenstive=0) {
	//if its case insenstive, convert strings into lower case
	if ($insenstive) {
		$haystack=strtolower($haystack);
		$needle=strtolower($needle);
	}
	//count number of occurances
	$count=substr_count($haystack,$needle);
	
	//first check if the needle exists in the haystack, return false if it does not
	//also check if asked nth is within the count, return false if it doesnt
	if ($count<1 || $nth > $count) return false;
	
	//run a loop to nth number of accurance
	for($i=0,$pos=0,$len=0;$i<$nth;$i++) {
		//get the position of needle in haystack
		//provide starting point 0 for first time ($pos=0, $len=0)
		//provide starting point as position + length of needle for next time
		$pos=strpos($haystack,$needle,$pos+$len);

		//check the length of needle to specify in strpos
		//do this only first time
		if ($i==0) $len=strlen($needle);
	}

	//return the number
	return $pos;
}

function strleft($s1, $s2) {
        return substr($s1, 0, strpos($s1, $s2));
}

/**
*  Merges two arrays of any dimension
*
*  This is the process' core!
*  Here each array is merged with the current resulting one
*
*  @access private
*  @author Chema Barcala Calveiro <shemari75@mixmail.com>
*  @param array $array  Resulting array - passed by reference
*  @param array $array_i Array to be merged - passed by reference
*/

function array_merge_2(&$array, &$array_i) {
	// For each element of the array (key => value):
	foreach ($array_i as $k => $v) {
		// If the value itself is an array, the process repeats recursively:
		if (is_array($v)) {
			if (!isset($array[$k])) {
			    $array[$k] = array();
			}
		array_merge_2($array[$k], $v);
 
		// Else, the value is assigned to the current element of the resulting array:
		} else {
			if (isset($array[$k]) && is_array($array[$k])) {
				$array[$k][0] = $v;
			} else {
				if (isset($array) && !is_array($array)) {
					$temp = $array;
					$array = array();
					$array[0] = $temp;
				}
				$array[$k] = $v;
			}
		}
	}
}

/**
*  Merges any number of arrays of any dimension
*
*  The arrays to be merged are passed as arguments to the function,
*  which uses an external function (array_merge_2) to merge each of them
*  with the resulting one as it's being constructed
*
*  @access public
*  @author Chema Barcala Calveiro <shemari75@mixmail.com>
*  @return array Resulting array, once all have been merged
*/

function array_merge_n() {
	// Initialization of the resulting array:
	$array = array();
	
	// Arrays to be merged (function's arguments):
	$arrays =& func_get_args();
	
	// Merging of each array with the resulting one:
	foreach ($arrays as $array_i) {
		if (is_array($array_i)) {
			array_merge_2($array, $array_i);
		}
	}

	return $array;
}



/**
* Returns the current url of the page
*/
function siteURL($removepagename = false) {
	$s = empty($_SERVER["HTTPS"]) ? ''
		: ($_SERVER["HTTPS"] == "on") ? "s"
		: "";
	$protocol = strleft(strtolower($_SERVER["SERVER_PROTOCOL"]), "/").$s;
	$port = ($_SERVER["SERVER_PORT"] == "80") ? "" : (":".$_SERVER["SERVER_PORT"]);
	
	if ($removepagename) {
		$pagename = $protocol . "://" . $_SERVER['SERVER_NAME'] . $port . substr($_SERVER['SCRIPT_NAME'], 0, strlen($_SERVER['SCRIPT_NAME']) - 9);
	} else {
		$pagename = $protocol . "://" . $_SERVER['SERVER_NAME'] . $port . $_SERVER['SCRIPT_NAME'];
	}
        return $pagename;
}

function thisPageURL($header = false, $absoluteurl = true) {
	global $page_id, $language;
	
	if ($_REQUEST["userlist_page"] != "") $extraparams .= "&amp;userlist_page=" . $_REQUEST["userlist_page"];
	if ($_REQUEST["user_id"] != "") $extraparams .= "&amp;user_id=" . $_REQUEST["user_id"];
	if ($_REQUEST["module"] != "") $extraparams .= "&amp;module=" . $_REQUEST["module"];
	if ($_REQUEST["action"] != "") $extraparams .= "&amp;action=" . $_REQUEST["action"];
	if ($_REQUEST["profile_mode"] != "") $extraparams .= "&amp;profile_mode=" . $_REQUEST['profile_mode'];
	if ($_REQUEST["language"] != "") $extraparams .= "&amp;language=$language";
	
	if ($absoluteurl) {
		$temp_absoluteurl = siteURL(true);
	} else {
		$temp_absoluteurl = "./";
	}
	
	$thispageurl = $temp_absoluteurl . "index.php?page_id=$page_id";
	
	if ($header) return str_replace("&amp;", "&", $thispageurl . $extraparams);
	return $thispageurl . $extraparams;
}

/**
 * Tab widget class
 */
class Tab {
	var $title = "Tab";
	var $url;
	var $current = false;
}
/**
 * TabBar widget class
 */
class TabBar {
	var $tabs = array();
	
	function addTab($title, $url = "", $current = false) {
		$tab = new Tab;
		$tab->title = $title;
		if ($url != "") $tab->url = $url;
		$tab->current = $current;
		$this->tabs[$title] = $tab;
	}
	
	function removeTab($title) {
		unset($this->tabs[$title]);
	}
	
	function renameTab($oldTitle, $newTitle) {
		$this->tabs[$newTitle] = $this->tabs[$oldTitle];
		$this->tabs[$newTitle]->title = $newTitle;
		unset($this->tabs[$oldTitle]);
	}
	function setCurrent($title) {
		foreach($this->tabs as $key => $value) {
			$this->tabs[$key]->current = false;
		}
		$this->tabs[$title]->current = true;
	}
	
	function renderTabBar() {
		$numberoftabs = count($this->tabs);
		$tabwidth = floor((98 - (2 * $numberoftabs)) / $numberoftabs);
		$compensation = 1 + ($numberoftabs * ($tabwidth - floor($tabwidth)));
		$tabbar = "
			<div style=\"width: 100%; margin: 0; padding: 0;\" class=\"tabbar\"><span style=\"display: inline-table; width: 1%; border: none; border-bottom: 1px solid black;\" class=\"tabbar_left\">&nbsp;</span>";
		foreach($this->tabs as $key => $value) {
			if(!$value->current) {
			$tabbar .= "<span style=\"display: inline-table; width: 1%; border: none; border-bottom: 1px solid black;\" class=\"tabbar_tab_left\">&nbsp;</span><a href=\"{$value->url}\" class=\"tabbar_tab\"><span style=\"display: inline-table; width: $tabwidth%; border: 1px solid black;\" class=\"tabbar_tab\">{$value->title}</span></a><span style=\"display: inline-table; width: 1%; border: none; border-bottom: 1px solid black;\" class=\"tabbar_tab_right\">&nbsp;</span>";
			} else {
			$tabbar .= "<span style=\"display: inline-table; width: 1%; border: none; border-bottom: 1px solid black;\" class=\"tabbar_tab_left_current\">&nbsp;</span><span style=\"display: inline-table; width: $tabwidth%; border: 1px solid black; border-bottom: none;\" class=\"tabbar_tab_current\">{$value->title}</span><span style=\"display: inline-table; width: 1%; border: none; border-bottom: 1px solid black;\" class=\"tabbar_tab_right_current\">&nbsp;</span>";
			}
		}
		$tabbar .= "<span style=\"display: inline-table; width: $compensation%; border: none; border-bottom: 1px solid black;\" class=\"tabbar_right\">&nbsp;</span></div>";
		return $tabbar;
	}
}

class CommandButton {
	var $title;
	var $command;
	var $url;
}

class TabWidget {
	var $tabbar;
	var $contents = "";
	var $commands;
	
	function TabWidget () {
		$this->tabbar = new TabBar;
	}
	function addCommand($command, $title, $url) {
		$this->commands[$command] = new CommandButton;
		$this->commands[$command]->title = $title;
		$this->commands[$command]->command = $command;
		$this->commands[$command]->url = $url;
	}
	
	function renderTabWidget() {
		$tabwidget = $this->tabbar->renderTabBar() . "<div style=\"width: 99%; margin: 0; padding: 0; border: 1px solid black; border-top: none;\" class=\"tabwidget\"><div class=\"wikirightalign\">";
		if (is_array($this->commands)) {
			foreach($this->commands as $key => $value) {
				$tabwidget .= "<a class=\"command\" title=\"" . $value->title . "\" href=\"" . $value->url . "\">[" . $value->command . "]</a>";
			}
		} else {
			$tabwidget .= "<small style=\"font-size: 1px;\">&nbsp;</small>";
		}
		$tabwidget .= "</div><div class=\"tabwidget_content\">" . $this->contents . "</div></div>";
		return $tabwidget;
	}
}
?>